/*
 * 2014/12 made by audin
 * http://mpu.seesaa.net
 *
 */
/******************************************************************************/
/*  Files to Include                                                          */
/******************************************************************************/
#include "sys.h"         /* system definition                        */
#include "uart.h"
#include "xprintf.h"

//*    This sets the MPIDE version number in the image header as defined in the linker script
const uint32_t __attribute__((section(".mpide_version"))) _verMPIDE_Stub = 0x01000309;    // assigns the build number in the header section in the image

/* 
   Config settings
   Main clock = 8MHz /2 * 24 /2   = 48MHz
   Peripheral clock = 48MHz /1    = 48MHz

   Configuration Bit settings
   SYSCLK = 48 MHz (8MHz / FPLLIDIV * FPLLMUL / FPLLODIV)
   PBCLK  = 48 MHz
   WDT OFF
   Other options are don't care
   */
#pragma config UPLLEN    = ON            // USB PLL Enabled
#pragma config FPLLMUL   = MUL_24        // PLL Multiplier
#pragma config UPLLIDIV  = DIV_2         // USB PLL Input Divider
#pragma config FPLLIDIV  = DIV_2         // PLL Input Divider
#pragma config FPLLODIV  = DIV_2         // PLL Output Divider
#pragma config FPBDIV    = DIV_1         // Peripheral Clock divisor
#pragma config FWDTEN    = OFF           // Watchdog Timer
#pragma config WDTPS     = PS1           // Watchdog Timer Postscale
#pragma config FCKSM     = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC  = OFF           // CLKO Enable
#pragma config POSCMOD   = HS            // Primary Oscillator
#pragma config IESO      = OFF           // Internal/External Switch-over
#pragma config FSOSCEN   = OFF           // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = FRCPLL         // Internal RC with PLL
//#pragma config FNOSC = PRIPL           // Ext OSC
#pragma config CP        = OFF           // Code Protect
#pragma config BWP       = OFF           // Boot Flash Write Protect
#pragma config PWP       = OFF           // Program Flash Write Protect
#pragma config ICESEL    = ICS_PGx1      // ICE/ICD Comm Channel Select
#pragma config DEBUG     = OFF           // Background Debugger Enable
#pragma config JTAGEN    = OFF           // JTAG Disabled
#pragma config FUSBIDIO  = OFF
#pragma config FVBUSONIO = OFF
#pragma config PMDL1WAY  = OFF
#pragma config IOL1WAY   = OFF

/******************************************************************************/
/* Macro Declaration                                                */
/******************************************************************************/
#define SYS_FREQ            FOSC
#define PB_DIV              1
#define PRESCALE            256
#define TOGGLES_PER_SEC     1000   /* 1msec Tick count */
#define T1_TICK             (SYS_FREQ/PB_DIV/PRESCALE/TOGGLES_PER_SEC)

#define LED1                BIT_15

/******************************************************************************/
/* Global Variable Declaration                                                */
/******************************************************************************/
volatile uint32_t ticks ;

/******************************************************************************/
/* millis()                                                                   */
/******************************************************************************/
uint32_t millis(void){
    return ticks;
}

/******************************************************************************/
/* Timer1 Interrupt                                                           */
/******************************************************************************/
void __ISR(_TIMER_1_VECTOR, ipl2) Timer1Handler(void)
{
    // clear the interrupt flag
    mT1ClearIntFlag();
    ticks++;
}

/******************************************************************************/
/* delay_ms()                                                                 */
/******************************************************************************/
void delay_ms(uint32_t ms ){
    uint32_t prev = millis();
    while( ( millis() - prev) < ms ){
        /* just wait */
    }
}

void timer_init(void){
    /* Timer1 init */
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_256, T1_TICK);
    /* set up the timer interrupt with a priority of 2 */
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);
}

/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/
int32_t main(void)
{
    uint32_t count = 0;
    /*
       SYS_CFG_WAIT_STATES (configures flash wait states from system clock)
       SYS_CFG_PB_BUS (configures the PB bus from the system clock)
       SYS_CFG_PCACHE (configures the pCache if used)
       SYS_CFG_ALL (configures the flash wait states, PB bus, and pCache)
       SYSTEMConfig(SYS_FREQ, SYS_CFG_ALL);
       SYSTEMConfigPerformance(SYS_FREQ);
       */
    SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES |  SYS_CFG_PCACHE);
    ANSELA = 0x0000;
    ANSELB = 0x0000;
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
    /* enable multi-vector interrupts */
    INTEnableSystemMultiVectoredInt();
    
    uart_init( UART1, 115200 );
    timer_init();

    /* Init LED port */
    mPORTBSetPinsDigitalOut( LED1 );
    mPORTBClearBits( LED1 );

    /* LED blinker */
    delay_ms( 500 );
    while(1) {
        delay_ms( 500 );
        mPORTBToggleBits( LED1 );
        xprintf("\n Hello !  0x%04X", count++);
    }
}

